@@ -1,14 +1,16 @@
 #include <math.h>
+#include <string.h>
 #include <Python.h>
 
 // Constants
 
 static const double jump_max_duration = 0.2;
-static const double jump_speed = 291 + (2 / 3);
-static const double jump_acc = 1458 + (1 / 3);
+static const double jump_speed = 291 + (2. / 3.);
+static const double jump_acc = 1458 + (1. / 3.);
 static const double boost_consumption = 33. + (1. / 3.);
 static const double max_speed = 2300;
 static const double simulation_dt = 1. / 30.;
+static const double angle_resistance_multiplier = 50;
 
 // Vector stuff
 
@@ -130,12 +132,20 @@ double dist(Vector vec1, Vector vec2)
     return magnitude(sub(vec1, vec2));
 }
 
-// Main lib
+// More math stuff
+
+struct post_correction
+{
+    Vector left;
+    Vector right;
+    _Bool swapped;
+};
 
 struct jump_shot
 {
     int found;
     Vector best_shot_vector;
+    struct post_correction targets;
 };
 
 struct aerial_shot
@@ -144,13 +154,7 @@ struct aerial_shot
     int fast;
     Vector ball_intercept;
     Vector best_shot_vector;
-};
-
-struct post_correction
-{
-    Vector left;
-    Vector right;
-    _Bool swapped;
+    struct post_correction targets;
 };
 
 struct post_correction correct_for_posts(Vector ball_location, Vector left_target, Vector right_target)
@@ -209,7 +213,7 @@ double throttle_acceleration(double car_velocity_x)
     return y0 + ((y1 - y0) / (x1 - x0)) * (car_velocity_x - x0);
 }
 
-double car_drive_to_target_simulation(double boost_accel, double car_to_target, double car_speed, int car_boost)
+double car_drive_to_target_simulation(double boost_accel, double car_to_target, double car_speed, int car_boost, double resistance)
 {
     double b = car_boost;
     double t = 0;
@@ -218,15 +222,20 @@ double car_drive_to_target_simulation(double boost_accel, double car_to_target,
 
     while (d > 50)
     {
+        if (v < 1410)
+            v += throttle_acceleration(v) * simulation_dt;
+        else
+            v -= resistance * simulation_dt;
+
         if (b > boost_consumption * simulation_dt && v < max_speed - (boost_accel * simulation_dt))
         {
             v += boost_accel * simulation_dt;
             b -= boost_consumption * simulation_dt;
         }
 
-        v += throttle_acceleration(v) * simulation_dt;
+        if (v > 0)
+            d -= v * simulation_dt;
 
-        d -= v * simulation_dt;
         t += simulation_dt;
     }
 
@@ -250,6 +259,8 @@ double car_turn_simulation(double turning_time, double car_speed)
     return v;
 }
 
+// Main lib
+
 int find_landing_plane(Vector car_location, Vector car_velocity, double gravity)
 {
     Vector l = car_location;
@@ -300,14 +311,8 @@ struct jump_viable_dat jump_is_viable(double T, double boost_accel, double dista
     double forward_angle = angle(flatten(direction), flatten(car_forward));
     double backward_angle = Py_MATH_PIl - forward_angle;
 
-    double forward_turn_time = forward_angle * 0.418;
-    double backward_turn_time = backward_angle * 0.418;
-
-    double forward_time = T - forward_turn_time;
-    double backward_time = T - backward_turn_time;
-
-    r.forward = forward_time > 0 && forward_time > car_drive_to_target_simulation(boost_accel, distance, car_turn_simulation(forward_turn_time, car_speed), car_boost);
-    r.backward = backward_time > 0 && distance < 1500 && backward_time > car_drive_to_target_simulation(boost_accel, distance, car_turn_simulation(backward_turn_time, car_speed), 0);
+    r.forward = T > car_drive_to_target_simulation(boost_accel, distance, car_speed, car_boost, forward_angle * angle_resistance_multiplier);
+    r.backward = distance < 1500 && T > car_drive_to_target_simulation(boost_accel, distance, car_speed, 0, backward_angle * angle_resistance_multiplier);
 
     return r;
 }
@@ -321,11 +326,11 @@ struct jump_shot parse_slice_for_jump_shot_with_target(double time_remaining, do
     struct opti_norm car_to_ball_norm = normalize(car_to_ball);
     Vector direction = car_to_ball_norm.vector;
 
-    struct post_correction pst_crrctn = correct_for_posts(ball_location, left_target, right_target);
-    if (!pst_crrctn.swapped)
+    r.targets = correct_for_posts(ball_location, left_target, right_target);
+    if (!r.targets.swapped)
     {
-        Vector left_vector = normalize(sub(pst_crrctn.left, ball_location)).vector;
-        Vector right_vector = normalize(sub(pst_crrctn.right, ball_location)).vector;
+        Vector left_vector = normalize(sub(r.targets.left, ball_location)).vector;
+        Vector right_vector = normalize(sub(r.targets.right, ball_location)).vector;
         r.best_shot_vector = clamp(direction, left_vector, right_vector);
 
         if (in_field(sub(ball_location, multiply(r.best_shot_vector, double_to_vector(best_shot_value))), 1))
@@ -370,10 +375,7 @@ _Bool double_jump_is_viable(double T, double boost_accel, double distance, Vecto
     if (z < 380 || z > 490)
         return 0;
 
-    double forward_turn_time = angle(flatten(direction), flatten(car_forward)) * 0.418;
-    double forward_time = T - forward_turn_time;
-
-    return forward_time > 0 && forward_time > car_drive_to_target_simulation(boost_accel, distance, car_turn_simulation(forward_turn_time, car_speed), car_boost);
+    return T > car_drive_to_target_simulation(boost_accel, distance, car_speed, car_boost, angle(flatten(direction), flatten(car_forward)) * angle_resistance_multiplier);
 }
 
 struct jump_shot parse_slice_for_double_jump_with_target(double time_remaining, double best_shot_value, Vector ball_location, Vector car_location, Vector car_forward, int car_boost, Vector left_target, Vector right_target, double boost_accel, double car_speed)
@@ -385,11 +387,11 @@ struct jump_shot parse_slice_for_double_jump_with_target(double time_remaining,
     struct opti_norm car_to_ball_norm = normalize(car_to_ball);
     Vector direction = car_to_ball_norm.vector;
 
-    struct post_correction pst_crrctn = correct_for_posts(ball_location, left_target, right_target);
-    if (!pst_crrctn.swapped)
+    r.targets = correct_for_posts(ball_location, left_target, right_target);
+    if (!r.targets.swapped)
     {
-        Vector left_vector = normalize(sub(pst_crrctn.left, ball_location)).vector;
-        Vector right_vector = normalize(sub(pst_crrctn.right, ball_location)).vector;
+        Vector left_vector = normalize(sub(r.targets.left, ball_location)).vector;
+        Vector right_vector = normalize(sub(r.targets.right, ball_location)).vector;
         r.best_shot_vector = clamp2D(direction, left_vector, right_vector);
 
         if (in_field(sub(ball_location, multiply(r.best_shot_vector, double_to_vector(best_shot_value))), 1) && find_slope(r.best_shot_vector, car_to_ball) > 0.5)
@@ -459,11 +461,7 @@ struct aerial_shot aerial_is_viable(double time_remaining, double hitbox_height,
 
     Vector velocity_estimate = add(vf, multiply(f, double_to_vector(boost_accel * (tau2 - tau1))));
 
-    _Bool enough_boost = boost_estimate < 0.95 * me.boost;
-
-    if (me.boost > 1000)
-        enough_boost = 1;
-
+    _Bool enough_boost = (me.boost > 1000) ? 1 : boost_estimate < 0.95 * me.boost;
     _Bool enough_time = fabs(ratio) < 0.95;
     _Bool enough_speed = magnitude(velocity_estimate) < 0.95 * max_speed;
 
@@ -492,19 +490,13 @@ struct aerial_shot aerial_is_viable(double time_remaining, double hitbox_height,
 
         Vector velocity_estimate = add(vf, multiply(f, double_to_vector(boost_accel * (tau2 - tau1))));
 
-        _Bool enough_boost;
-
-        if (me.boost > 1000)
-            enough_boost = 1;
-        else
-            enough_boost = boost_estimate < 0.95 * me.boost;
-
+        _Bool enough_boost = (me.boost > 1000) ? 1 : boost_estimate < 0.95 * me.boost;
         _Bool enough_time = fabs(ratio) < 0.95;
         _Bool enough_speed = magnitude(velocity_estimate) < 0.95 * max_speed;
 
         if (enough_speed && enough_boost && enough_time)
             r.found = 1;
-        r.fast = 0;
+            r.fast = 0;
     }
 
     return r;
@@ -518,11 +510,11 @@ struct aerial_shot parse_slice_for_aerial_shot_with_target(double time_remaining
     Vector car_to_ball = sub(ball_location, me.location);
     Vector direction = normalize(car_to_ball).vector;
 
-    struct post_correction pst_crrctn = correct_for_posts(ball_location, left_target, right_target);
-    if (!pst_crrctn.swapped)
+    r.targets = correct_for_posts(ball_location, left_target, right_target);
+    if (!r.targets.swapped)
     {
-        Vector left_vector = normalize(sub(pst_crrctn.left, ball_location)).vector;
-        Vector right_vector = normalize(sub(pst_crrctn.right, ball_location)).vector;
+        Vector left_vector = normalize(sub(r.targets.left, ball_location)).vector;
+        Vector right_vector = normalize(sub(r.targets.right, ball_location)).vector;
 
         r.best_shot_vector = clamp(direction, left_vector, right_vector);
         r.ball_intercept = sub(ball_location, multiply(r.best_shot_vector, double_to_vector(best_shot_value)));
@@ -566,6 +558,8 @@ struct aerial_shot parse_slice_for_aerial_shot(double time_remaining, double bes
     return r;
 };
 
+// Linking C functions to Python
+
 static PyObject *method_parse_slice_for_jump_shot_with_target(PyObject *self, PyObject *args)
 {
     int car_boost;
@@ -587,7 +581,7 @@ static PyObject *method_parse_slice_for_jump_shot_with_target(PyObject *self, Py
 
     struct jump_shot data_struct = parse_slice_for_jump_shot_with_target(time_remaining, best_shot_value, ball_location, car_location, car_forward, car_boost, left_target, right_target, boost_accel, car_speed);
 
-    return Py_BuildValue("{s:i,s:(ddd)}", "found", data_struct.found, "best_shot_vector", data_struct.best_shot_vector.x, data_struct.best_shot_vector.y, data_struct.best_shot_vector.z);
+    return Py_BuildValue("{s:i,s:(ddd),s:((ddd)(ddd))}", "found", data_struct.found, "best_shot_vector", data_struct.best_shot_vector.x, data_struct.best_shot_vector.y, data_struct.best_shot_vector.z, "targets", data_struct.targets.left.x, data_struct.targets.left.y, data_struct.targets.left.z, data_struct.targets.right.x, data_struct.targets.right.y, data_struct.targets.right.z);
 };
 
 static PyObject *method_parse_slice_for_jump_shot(PyObject *self, PyObject *args)
@@ -648,7 +642,7 @@ static PyObject *method_parse_slice_for_double_jump_with_target(PyObject *self,
 
     struct jump_shot data_struct = parse_slice_for_double_jump_with_target(time_remaining, best_shot_value, ball_location, car_location, car_forward, car_boost, left_target, right_target, boost_accel, car_speed);
 
-    return Py_BuildValue("{s:i,s:(ddd)}", "found", data_struct.found, "best_shot_vector", data_struct.best_shot_vector.x, data_struct.best_shot_vector.y, data_struct.best_shot_vector.z);
+    return Py_BuildValue("{s:i,s:(ddd),s:((ddd)(ddd))}", "found", data_struct.found, "best_shot_vector", data_struct.best_shot_vector.x, data_struct.best_shot_vector.y, data_struct.best_shot_vector.z, "targets", data_struct.targets.left.x, data_struct.targets.left.y, data_struct.targets.left.z, data_struct.targets.right.x, data_struct.targets.right.y, data_struct.targets.right.z);
 };
 
 static PyObject *method_parse_slice_for_double_jump(PyObject *self, PyObject *args)
@@ -701,7 +695,7 @@ static PyObject *method_parse_slice_for_aerial_shot_with_target(PyObject *self,
 
     struct aerial_shot data_struct = parse_slice_for_aerial_shot_with_target(time_remaining, best_shot_value, boost_accel, gravity, ball_location, me, left_target, right_target);
 
-    return Py_BuildValue("{s:i,s:(ddd),s:(ddd),s:i}", "found", data_struct.found, "ball_intercept", data_struct.ball_intercept.x, data_struct.ball_intercept.y, data_struct.ball_intercept.z, "best_shot_vector", data_struct.best_shot_vector.x, data_struct.best_shot_vector.y, data_struct.best_shot_vector.z, "fast", data_struct.fast);
+    return Py_BuildValue("{s:i,s:(ddd),s:(ddd),s:((ddd)(ddd)),s:i}", "found", data_struct.found, "ball_intercept", data_struct.ball_intercept.x, data_struct.ball_intercept.y, data_struct.ball_intercept.z, "best_shot_vector", data_struct.best_shot_vector.x, data_struct.best_shot_vector.y, data_struct.best_shot_vector.z, "targets", data_struct.targets.left.x, data_struct.targets.left.y, data_struct.targets.left.z, data_struct.targets.right.x, data_struct.targets.right.y, data_struct.targets.right.z, "fast", data_struct.fast);
 };
 
 static PyObject *method_parse_slice_for_aerial_shot(PyObject *self, PyObject *args)
